--- BehaviorBase module
--- Created by: Mark Fainstein at: 04/19/15
-- @module behaviorBase

function TTP_BehaviorBase(self)

 local PubSub = TTP_PubSub

 self.publishEvent = PubSub.publishEvent
 self.subscribeEvent = PubSub.subscribeEvent
 self.unsubscribeEvent = PubSub.unsubscribeEvent

 if not self._behaviorTarget then
  services:debugMessageBox("BehaviorBase", "issue with self passed to BehaviorBase module")
 end

  --- gets the id of this behavior
  -- @function getId
	function self.getId()
    return self._behaviorTarget:getTargetId()
 	end

  --- get an object
-- @function getObject
-- @param id
 function self.getObject(id)
  if not id then return end
  if self.getId() == id then return self._behaviorTarget end
  local obj = self._behaviorTarget:getTargetById(id)
  return obj
 end

 -- get a behavior, from a specific object
 -- @function getBehavior
 -- @param id
 -- @param behaviorName
  function self.getBehavior(id, behaviorName)
  local object = self.getObject(id)
  if object then
   return object:getBehavior(behaviorName)
  end
 end

 --- get an application behavior (which is attached to ttScenes)
 -- @function getApplicationBehavior
 -- @param behaviorName
 function self.getApplicationBehavior(behaviorName)
  if self.getId() == "ApplicationObject" then return self.getBehavior("ApplicationObject", behaviorName) end
  -- temporary workaround (!!!)
  -- TODO: remove this after it has been fixed

  local applicationBehavior = _G[behaviorName.."_ApplicationObject"]

  if applicationBehavior  then
      return applicationBehavior
  else
      services:debugMessageBox("Missing Application Behavior !!!", "Can't find app behavior"..behaviorName.."_ApplicationObject")
      return
  end
 end


 --- creates and returns a param container
 -- @function ttFuncParams
 -- @param ... list of parameters
 -- @return paramsContainer
 local function ttFuncParams(...)
     local paramsContainer = ParamsContainer:new()
     local params = {...}
     for k,v in pairs (params) do
         paramsContainer:add(v)
     end
     return paramsContainer
 end

 function self.runFunc(object, funcName, ...)
     local ttFuncParams = TTP_Utils.ttFuncParams
     local xmlEngineComponent = object:getComponent("IXmlEngineComponent")
     xmlEngineComponent:runttFunction(funcName, ttFuncParams(...))
     return true
 end

 --- gets value from persistency
 -- @function getValue
 function self.getPersistValue(persistancyPackage, fieldValue)
     if not persistancyPackage or not fieldValue then
         services:debugMessageBox("BehaviorBase", "Method getPersistValue is missing parameter!")
     end
     local _val = persistancyPackage:getValue(fieldValue)
     return _val
 end

 --- sets value to persistency
 -- @function setValue
 function self.setPersistValue(persistancyPackage, fieldValue, val)
     if not persistancyPackage or not fieldValue or not val then
         services:debugMessageBox("BehaviorBase", "Method setPersistValue is missing parameter!")
     end
     persistancyPackage:setValue(fieldValue, val)
 end


 --- gets value from GameData.plist
 -- @function getValue
 function self.getScoreValue(fieldValue)
     if not fieldValue  then
         services:debugMessageBox("BehaviorBase", "Method getScoreValue is missing parameter!")
     end
     if not scores:exists(fieldValue) then
         return 0
     else
         return scores:getScore(fieldValue):getValue()
     end
 end

 --- sets value to GameData.plist
 -- @function setScoreValue
 function self.setScoreValue(fieldValue, val)
     if not fieldValue or not val then
         services:debugMessageBox("BehaviorBase", "Method setScoreValue is missing parameter!")
     end
     scores:getScore(fieldValue):setValue(val)
 end

 return self
end





