--
-- Created by: shalmuy at 1/3/16 Time: 1:38 PM
--

TTP_TimeManager = TTP_TimeManager or {}
TTP_TimeManager.Timeout = (function()
    local self = {}
    local private = {} --- "priv" contains private fields! This trick allows me not to think, which local field must be declared before another, so now I can declare them not in a sequence of visibility to each other, but the way I feel like, for example to group them the most sensibe way.


    --- hiding alarm from anyone!
    package.preload.alarm()
    assert(alarm, '"alarm" function does not exist, "TimeManager.Timeout" package cannot work without it at all')
    private.alarm=alarm
    alarm=nil


    private.IDcounter = 0

    private.pQueue = TTP_Utils.heap.new(true)

    function private.queueIsEmpty() return private.pQueue.isempty() end

    --- actually adds the event the the queue, used by public functions
    function private.push(delaySecondsAbs, delaySecondsRel, func, isLooped, id, arguments)
        private.pQueue.push({
            secs = delaySecondsAbs,
            secsRel = delaySecondsRel,
            func = func,
            arg = arguments,
            id = id,
            isLooped = isLooped
        }, delaySecondsAbs)
    end

    --- looks for the nearest event by time.
    function private.closestEventSecs()
        local obj, secs = self.peekClosestEvent()
        return secs or 0
    end

    --- had to publish this function, because TimeManager needs it.
    function self.peekClosestEvent()
        if private.queueIsEmpty() then return nil, nil end
        return private.pQueue.peek();
    end


    --- returns the closest event object
    function private.shift()
        if private.queueIsEmpty() then return nil end
        local obj, secs = private.pQueue.pop()
        return obj
    end


    --- turns "number of seconds, relative to now" to an absolute value.
    function private.rel2abs(relSeconds) return relSeconds + os.time() end

    --- turns an absolute time value to relative number of seconds as of current moment.
    function private.abs2rel(absSeconds) return absSeconds - os.time() end

    local tickCount = 0 --- counter only for debugging
    local function tick()
        print('tick: '..tickCount) tickCount=tickCount+1

        if not private.queueIsEmpty() then
            local calls={};
            local secNow = private.closestEventSecs()
            while secNow == private.closestEventSecs() do
                local o = private.shift()
                if o then
--                    print('   tick: collecting event N'..o.id..', set to '..o.secsRel..'(now: '..secNow..') seconds, params num: '..#o.arg)
                    table.insert(calls, o)
                end
            end
            if #calls then
--                print('collected '..#calls..' calls for this tick, firing them all:')
                for k, o in pairs(calls) do
--                    print('/////////////////////////////// firing function '..k..'...');
                    o.func(unpack(o.arg))
                    if o.isLooped then
                        self.loop(o.secsRel, o.func, unpack(o.arg))
                    end
                end
            else
--                print('idle, nothing to fire...')
            end
        else
--            print('idle, nothing to fire...')
        end
        private.alarm(1, tick)
    end


    local onlyOneTick =false
    --- this function makes sure I won't run the "tick" twice:
    -- @function runTickOnce
    local function runTickOnce()
        if not onlyOneTick then
            onlyOneTick =true
            tick()
        end
    end

    --- delays the function call to a particular number of seconds.
    -- @function delay
    -- @param delaySeconds the number of seconds to delay for
    -- @param func the function to call after "delaySeconds" is out
    -- @return an ID in case if cancellation will be neeeded later
    function self.delay(delaySeconds, func, ...)
        private.IDcounter = private.IDcounter + 1
        private.push(private.rel2abs(delaySeconds), delaySeconds, func, false, private.IDcounter, arg);
        runTickOnce();
        return private.IDcounter;
    end

    --- makes func loop every delaySeconds
    -- @function loop
    -- @param delaySeconds the number of seconds to loop by
    -- @param func the function to call after each delaySeconds
    -- @return an ID in case if cancellation will be neeeded later
    function self.loop(delaySeconds, func, ...)
        error('Not implemented yet.');
    end



    --- cancel the previously delayed function call
    -- @param id the unique number, returned by "delay" function
    function self.cancel(id)
        for i, event in pairs(private.pQueue.getheap()) do
            if event.id == id then
                private.pQueue.remove(i)
                return true
            end
        end
    end


    return self;
end)()

